# -*- coding: utf-8 -*-

# @Time     : 2021/10/30 0:22
# @Software : PyCharm
# @License  : GNU General Public License v3.0
# @Author   : xxx
import pyximport
from mgetool.tool import tt

pyximport.install()
import magic
import magic2
a = magic.A()
b = magic2.A()

tt.t
a.foo(1000)
tt.t
b.foo(1000)
tt.t
tt.p